package ru.dusystems.aopportlets.genclasses;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ITreeSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;

/**
 * @author omikheev
 * @version $Id: GenClassesAction.java,v 1.3 2008/09/16 21:14:11 cvsmihel Exp $
 */
public class GenClassesAction implements IObjectActionDelegate {

    private Shell shell;

    /**
     * Constructor for Action1.
     */
    public GenClassesAction() {
        super();
    }

    /**
     * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
     */
    public void setActivePart(IAction action, IWorkbenchPart targetPart) {
        shell = targetPart.getSite().getShell();
    }

    /**
     * @see org.eclipse.ui.IActionDelegate#run(IAction)
     */
    public void run(IAction action) {
        XMLReader parser = null;
        try {
            parser = XMLReaderFactory.createXMLReader();
            parser.setFeature("http://xml.org/sax/features/namespaces", true);
            parser.setFeature("http://xml.org/sax/features/namespace-prefixes", true);
            AopSaxHandler handler = new AopSaxHandler();
            parser.setContentHandler(handler);
            ITreeSelection selection = (ITreeSelection) PlatformUI.getWorkbench().getActiveWorkbenchWindow().getSelectionService().getSelection();
            final Object oFileAop = selection.getFirstElement();
            InputSource isrc = new InputSource(((IFile) oFileAop).getContents());
            parser.parse(isrc);
            String sSrcPath = null;
            IProject prj = ((IResource) oFileAop).getProject();
            IJavaProject jprj = JavaCore.create(prj);
            IClasspathEntry[] cpea = jprj.getResolvedClasspath(true);
            for (IClasspathEntry cpe : cpea) {
                if (cpe.getEntryKind() == IClasspathEntry.CPE_SOURCE) {
                    sSrcPath = cpe.getPath().makeRelative().toString();
                    // the first token is project name, need to strip
                    sSrcPath = sSrcPath.substring(sSrcPath.indexOf('/') + 1);
                    break;
                }
            }
            ClassSrcGenerator csg = new ClassSrcGenerator();
            Map<String, Set> mpPortletModes = handler.getPortletModes();
            for (String sPortletClassName : mpPortletModes.keySet()) {
                String sPkgName = sPortletClassName.substring(0, sPortletClassName.lastIndexOf('.')).replace('.', '/');
                createFolders(prj, sSrcPath + "/" + sPkgName);
                Set stModes = mpPortletModes.get(sPortletClassName);
                for (Object stMode : stModes) {
                    String sMode = (String) stMode;
                    IFile file = prj.getFile(sSrcPath + "/" + sPortletClassName.replace('.', '/') + sMode + "Aspect.java");
                    file.create(new ByteArrayInputStream(csg.getClassSource(sPortletClassName + sMode + "Aspect", sMode)
                            .getBytes()), true, null);
                }
            }
        } catch (SAXException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (CoreException e) {
            e.printStackTrace();
        }
    }

    private void createFolders(IProject prj, String path) {
        StringTokenizer st = new StringTokenizer(path, "/");
        StringBuilder sb = new StringBuilder();
        while (st.hasMoreTokens()) {
            sb.append(st.nextToken());
            IFolder fld = prj.getFolder(sb.toString());
            if (!fld.exists()) {
                try {
                    fld.create(true, true, null);
                } catch (CoreException e) {
                    e.printStackTrace();
                }
            }
            sb.append("/");
        }
    }

    /**
     * @see org.eclipse.ui.IActionDelegate#selectionChanged(IAction, ISelection)
     */
    public void selectionChanged(IAction action, ISelection selection) {
    }
}
